# Function that for one particular iteration loads the un-subsetted dataset, 
# permutes the target variable, samples the respective number of covariates 
# and calculates the cross-validation error of the respective method.
#
# Input parameters:
#
# z = line of "scenariogrid" corresponding to a particular iteration (see "RealDataStudy.R")

performAnalysis <- function(z) {

  set.seed(scenariogrid$seed[z])

  load("./FishingPaper/Datasetfiles/E-MTAB-1532.Rda")  

  y <- sample(y)
  X <- X[,sample(1:ncol(X), size=scenariogrid$nvar[z])]
  
  errorandtime <- try({applyClassificator(x=X, y=y, method=scenariogrid$method[z])})
  
  if(class(errorandtime)!="try-error"){
    return(list(iter=scenariogrid$iter[z], nvar=scenariogrid$nvar[z],
      method=scenariogrid$method[z], error=errorandtime$error, 
      time=errorandtime$time))
  }
  else{
    return(list(iter=scenariogrid$iter[z], nvar=scenariogrid$nvar[z],
      method=scenariogrid$method[z], error=errorandtime, 
      time=0))    
  }
}





# Function that for one particular iteration loads the subsetted dataset, 
# permutes the target variable, samples the respective number of covariates 
# and calculates the cross-validation error of the respective method.
#
# Input parameters:
#
# z = line of "scenariogrid" corresponding to a particular iteration (see "RealDataStudySubset.R")

performAnalysisSubset <- function(z) {

  set.seed(scenariogrid$seed[z])

  load("./FishingPaper/Datasetfiles/E-MTAB-1532Subset.Rda")  

  y <- sample(y)
  X <- X[,sample(1:ncol(X), size=scenariogrid$nvar[z])]
  
  errorandtime <- try({applyClassificator(x=X, y=y, method=scenariogrid$method[z])})
  
  if(class(errorandtime)!="try-error"){
    return(list(iter=scenariogrid$iter[z], nvar=scenariogrid$nvar[z],
      method=scenariogrid$method[z], error=errorandtime$error, 
      time=errorandtime$time))
  }
  else{
    return(list(iter=scenariogrid$iter[z], nvar=scenariogrid$nvar[z],
      method=scenariogrid$method[z], error=errorandtime, 
      time=0))    
  }
}






# Function that performs cross-validation of the passed method using
# the passed dataset:
#
# Input parameters:
#
# x = covariate matrix
# y = target variable
# method = character. classification method.

applyClassificator <- function(x, y, method) {

  require("CMA")
  
  cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = 5, niter = 1)

  if(method=="DLDA") {
  
    start <- Sys.time()
	
    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = dldaCMA)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))
 
  }
  
  if(method=="ElNet") {

    start <- Sys.time()
	
    norm.fraction.grid <- seq(from=0.1, to=0.9, length=5)
	alpha.grid <- c(0.01, 0.3, 0.5, 0.7, 0.99)

    tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = ElasticNetCMA, 
      grids = list(norm.fraction = norm.fraction.grid, alpha = alpha.grid))

    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = ElasticNetCMA, tuneres = tunenestedcv)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))
	
  }  
    
  if(method=="kNN") {

    start <- Sys.time()
	
    k.grid <- 1:10

    tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = knnCMA, 
      grids = list(k = k.grid))

    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = knnCMA, tuneres = tunenestedcv)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))
  
  }
  
  if(method=="pkNN") {

    start <- Sys.time()
	
    k.grid <- 1:10

    tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = pknnCMA, 
      grids = list(k = k.grid))

    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = pknnCMA, tuneres = tunenestedcv)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))

  }    

  if(method=="penLogReg") {

    start <- Sys.time()
	
    lambda.grid <- 2^{-4:4}

    tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = plrCMA, 
      grids = list(lambda = lambda.grid))

    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = plrCMA, tuneres = tunenestedcv)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))

  }   
  
  if(method=="RanFor") {

    start <- Sys.time()
	
    eval(parse(text=paste("nestedcrossval <- classification(x, y, learningsets = cvdatasets, classifier = rfCMA, mtry=", floor(sqrt(ncol(x))), ")", sep="")))

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))

  }     
  
  if(method=="shrinkLDA") {

    start <- Sys.time()
	
    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = shrinkldaCMA)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))
	
  }	
  
  if(method=="SVMlinear") {

    start <- Sys.time()
	
    cost.grid <- c(0.1, 1, 5, 10, 50, 100, 500)
  
    tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = svmCMA, 
      grids = list(cost = cost.grid), kernel="linear")

    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = svmCMA, tuneres = tunenestedcv, kernel="linear")

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))
	
  }	    
  
  if(method=="SVMradial") {

    start <- Sys.time()
	
    cost.grid <- c(0.1, 1, 5, 10, 50, 100, 500)

    eval(parse(text=paste("tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = svmCMA, grids = list(cost = cost.grid), kernel=\"radial\", gamma=", ifelse(1/ncol(x) > 0.01, 1/ncol(x), 0.01), ")", sep="")))

    eval(parse(text=paste("nestedcrossval <- classification(x, y, learningsets = cvdatasets, classifier = svmCMA, tuneres = tunenestedcv, kernel=\"radial\", gamma=", ifelse(1/ncol(x) > 0.01, 1/ncol(x), 0.01), ")", sep="")))

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))
	
  }	    
  
  if(method=="cboost") {

    start <- Sys.time()
	
    mstop.grid <- c(50, 100, 200, 500, 1000)

    tunenestedcv <- tune(x, y, learningsets = cvdatasets, classifier = compBoostCMA, 
      grids = list(mstop = mstop.grid))

    nestedcrossval <- classification(x, y, learningsets = cvdatasets, 
      classifier = compBoostCMA, tuneres = tunenestedcv)

    misclassnestedcv <- evaluation(nestedcrossval, measure = "misclassification")

    error <- mean(misclassnestedcv@score)

    stop <- Sys.time()

    time <- stop - start
	
    return(list(error=error, time=as.double(time, units="secs")))

  } 
  

}
